/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 2011,2012
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 

#ifndef OptimReportingContext_H
#define OptimReportingContext_H

/*
 * File: OptimReportingContext.h
 *
 * Function: Main class for Console Progress Display
 *
 * --------------------------------------------------------------------------
 *
 * RTC/Jazz change log
 *
 *
 * 15 Jul 2013 Shilpa 9.1.0.5 WI 52348
 * Added new Class for Optim Report Intiative instantiated for each Action
 *
 *
 *
 *
 * Initial revision.
 * Author:   RAP
 * Created:  6/19/13
*/
#include "RtErrors.h"          // Base class for error checking
#include "ActnPort.h"          // ActnPort class for report
#include "ActnRept.h"          // Action Report Class
#include "RtCommon.h"          // Our common includes
#include "RtDir.h"             // PST Dir class for Audit
#include "RtMisc.h"            // RT Miscellaneous class
#include "RTVER.H"             // RT Version details
#include "RTBR.H"              // Rt Browse class
#include "RtLicens.h"          // License class
#include "convmcsv.h"
#include "convcommon.h"        // Convert File format common header.


/*----------------------------------------------------------------------
 * Prepare for import or exporting
 *---------------------------------------------------------------------*/
#ifdef OptimReportingContext_CPP
   #define OptimReportingContext_EXPORT __declspec(dllexport)
#else
   #define OptimReportingContext_EXPORT __declspec(dllimport)
#endif

// Definitions for message types
#define ORC_RUNTIME_NAME               L"OPTIM"
#define ORC_RUNTIME_VERSION            L"x.x.x.x"
#define ORC_SOLUTION_NAME              L"IBM InfoSphere Optim Data Growth Solution"
#define ORC_SOLUTION_VERSION           L"x.x"
#define ORC_OPTDIR_TYPE                L"OPT_DIR"
#define ORC_TABLE_STRING               L"TABLE"
#define ORC_INDEX_STRING               L"INDEX"
#define ORC_TABLE_MAP                  L"tableMapName"
#define ORC_EXTRACT_FILE               L"extractFile"
#define ORC_CONTROL_FILE               L"controlFile"
#define ORC_DDL_STRING                 L"ddl"
#define ORC_COMPARE_FILE               L"compareFile"
#define ORC_NAME_STRING                L"Name"
#define ORC_DELIM_SCORE                L"-"
#define ORC_EMPTY_LSTRING              L" "


#define ORC_EXTRACT_DATA               _T("DATA")
#define ORC_EXTRACT_OBJECTS            _T("OBJECTS")
#define ORC_EXTRACT_BOTH               _T("BOTH")
#define ORC_REPORT_NAME                _T("Optim-Report")
#define ORC_OBJECT_TABLES              _T("Tables")
#define ORC_OBJECT_PRIMARYKEYS         _T("PrimaryKeys")
#define ORC_OBJECT_RELATIONSHIPS       _T("Relationships")
#define ORC_OBJECT_INDEXES             _T("Indexes")
#define ORC_OBJECT_ALIASES             _T("Aliases")
#define ORC_OBJECT_SYNOMNS             _T("Synomns")
#define ORC_OBJECT_FUNCTIONS           _T("Functions")
#define ORC_OBJECT_PACKAGES            _T("Packages")
#define ORC_OBJECT_PROCEDURES          _T("Procedures")
#define ORC_OBJECT_SEQUENCES           _T("Sequences")
#define ORC_OBJECT_TRIGGERS            _T("Triggers")
#define ORC_OBJECT_VIEWS               _T("Views")
#define ORC_OBJECT_DEFAULTS            _T("Defaults")
#define ORC_OBJECT_RULES               _T("Rules")
#define ORC_OBJECT_UDTS                _T("UDTs")
#define ORC_OBJECT_ASSEMBLIES          _T("Assemblies")
#define ORC_OBJECT_PARTITIONFUNCTIONS  _T("PartitionFunction")
#define ORC_OBJECT_PARTITIONSCHEMA     _T("PartitionSchema")
#define ORC_OBJECT_EXTENDEDOBJECTS      _T("ExtendedObjects")
#define ORC_DEFAULT_VALUE 0
#define ORC_MAX_LEN 1024
#define ORC_DATE_LEN 50
#define ORC_NAME_LEN 256

// OptimReportingConcept Structure Values
struct optimReportingMessage {
   struct optimReportingMessage *   pNext;
	long long 		id;
	wchar_t 		   message[ORC_MAX_LEN];
	int				type; // Use ORC_MESSAGE_TYPE*
};
typedef struct optimReportingMessage OptimReportingMessage_t;


struct optimReportingNameValuePair {
   struct optimReportingNameValuePair *   pNext;
	wchar_t 			name[ORC_NAME_LEN];
	wchar_t			value[ORC_NAME_LEN];
};
typedef struct optimReportingNameValuePair OptimReportingNameValuePair_t;

struct optimReportingNumberPair {
   struct optimReportingNumberPair *   pNext;
	long long		number1;
	long long		number2;
};
typedef struct optimReportingNumberPair OptimReportingNumberPair_t;

struct optimReportingEntity {
    struct optimReportingEntity *   pNext;
    long long		   recordCount;
    long long		   compareUnmatched;
    long long		   compareEqual;
    long long		   compareDifferent;
    long long		   compareDuplicate;
    long long		   convertCsvLimitRowCount;
    long long		   convertCsvLimitFileSize;
    long long		   convertCsvLimitFileCount;
    long long 		   insertInserted;
    long long		   insertUpdated;
    long long		   skipped;
    long long		   failed;
    double			   extractAverageRowLength;
    double			   extractMaxLobLength;
    wchar_t			   status[ORC_NAME_LEN];
    wchar_t			   name[ORC_NAME_LEN];
    wchar_t			   convertCsvFileName[ORC_NAME_LEN];
    int				   type;
    int				   extractCompressionValue;
    long long        deleteNotFound;                        
    unsigned char 	extractCompressionEnabled;             // boolean
    unsigned char 	extractLob;                            // boolean
};
typedef struct optimReportingEntity OptimReportingEntity_t;

typedef struct optimReportingExtendedObjects
{
   struct optimReportingExtendedObjects *   pNext;
   wchar_t        name[ORC_NAME_LEN];              	                     // name   
   int            type;                                     // type
   int            count;                                    // count
   wchar_t        modifyDate[ORC_DATE_LEN];               	            // modifyDate   
};
typedef struct optimReportingExtendedObjects OptimReportingExtendedObjects_t;

typedef struct optimReportingIndexImpAnalysis
{
   struct optimReportingIndexImpAnalysis *   pNext;
   wchar_t        name[ORC_NAME_LEN];              	                     // name   
   wchar_t        type[ORC_NAME_LEN];                                    // type      
   wchar_t        status[ORC_NAME_LEN];               	                  // status
   long long      storageSavings;                           // storagesavings   
};
typedef struct optimReportingIndexImpAnalysis OptimReportingIndexImpAnalysis_t;

typedef struct optimReportingTableImpAnalysis
{
   struct optimReportingTableImpAnalysis *   pNext;
   wchar_t        name[ORC_NAME_LEN];              	                   // name   
   wchar_t        type[ORC_NAME_LEN];                                  // type      
   wchar_t        status[ORC_NAME_LEN];               	                // status
   long long      storageSavings;                         // storagesavings
   unsigned int	numIndexAnalysis;                       // Number of index Analysis in ImpactAnalysis array
   OptimReportingIndexImpAnalysis_t *pIndexAnalysis;
};
typedef struct optimReportingTableImpAnalysis OptimReportingTableImpAnalysis_t;

typedef struct optimReportingJoinControl
{
   struct optimReportingJoinControl *   pNext;
   wchar_t        tableName[ORC_NAME_LEN];                              // tableName            
   wchar_t        joinedTable[ORC_NAME_LEN];                            // parent
};
typedef struct optimReportingJoinControl OptimReportingJoinControl_t;

struct optimReportingRelationship {
   struct optimReportingRelationship *   pNext;
	wchar_t 		   relationshipName[ORC_NAME_LEN];
	wchar_t			parentName[ORC_NAME_LEN];
	wchar_t			childName[ORC_NAME_LEN];
	int				parentAccessType;
	int				parentKeyLimit;
	int				childAccessType;
	int				childKeyLimit;
};
typedef struct optimReportingRelationship OptimReportingRelationship_t;

struct optimReportingStatistics {
   struct optimReportingStatistics *   pNext;
	long long		estimatedRowCount;
	long long		fetchBuffer;
	long long		foreignKeyIndex;
	long long		foreignKeyNoIndex;
	long long		primaryKeyIndex;
	long long		primaryKeyNoIndex;
	long long		dbDataPerSecond;
	long long		dbProcessTime;
	long long		dbRowsFetched;
	long long		dbRowsPerSecond;
	long long		dbRowsWritten;
	long long		dbTimeInAct;
	long long		dbTimeInDB;
	wchar_t			name[ORC_NAME_LEN];
	wchar_t			dbmsVendor[ORC_NAME_LEN];
	wchar_t			dbmsVersion[ORC_NAME_LEN];
	wchar_t			relName[ORC_NAME_LEN];
	wchar_t			relSql[ORC_NAME_LEN];
	int				childStrategy;
	int				columnCount;
	int				databaseConnections;
	int				method;
	int				parentStrategy;
	int				rowLength;
	int				stepNumber;
   wchar_t			connectionMode[ORC_NAME_LEN];
   wchar_t			whereClause[ORC_NAME_LEN];
   wchar_t			rowid[ORC_NAME_LEN];  
	int				relAccess;
	int				relDb2Cost;
	int				relDb2ScanCost;
	int			   relDirection;
	int				relKeyLength;
	int				relKeyPerCur;
	int				relLookupKey;
	int				relType;   
	int				dbAccessType;
	int				dbKeysPer;
	int				dbOpenCursor;
	unsigned char	cycle; //boolean
	unsigned char	lobs;
	unsigned char	selectUniqueRecord;
	unsigned char	relIndex;
   wchar_t			compTableName[ORC_NAME_LEN];
   wchar_t			compRatio[ORC_NAME_LEN];
};
typedef struct optimReportingStatistics OptimReportingStatistics_t;

struct optimReportingDdl {
   wchar_t        ddl[ORC_MAX_LEN];
   int            type;
};
typedef struct optimReportingDdl OptimReportingDdl_t;

struct optimReportingContext { // Optimize this for layout later!!!!
    long long		                        errorCount; // We want a 64-bit value here. Unsigned is fine. Signed will do.
    long long		                        infoCount;
    long long 		                        warningCount;
    long long 		                        timeElapsed;
    long long 		                        totalRows;
    long long		                        totalRowsFirstPass;
    long long		                        totalRowsInError;
    long long		                        totalTables;
    wchar_t 		                        reportFile[ORC_NAME_LEN];
    wchar_t 		                        htmlReportFile[ORC_NAME_LEN];
    wchar_t			                        optimDirectoryName[ORC_NAME_LEN];
    wchar_t			                        optimDirectoryDBMSType[ORC_NAME_LEN];
    wchar_t			                        optimDirectoryDBMSVendor[ORC_NAME_LEN];
    wchar_t			                        optimDirectoryDBMSVersion[ORC_NAME_LEN];
    wchar_t			                        optimSolutionName[ORC_NAME_LEN];
    wchar_t			                        optimSolutionVersion[ORC_NAME_LEN];
    wchar_t			                        optimRuntimeName[ORC_NAME_LEN];
    wchar_t			                        optimRuntimeVersion[ORC_NAME_LEN];
    wchar_t			                        clientUserId[ORC_NAME_LEN];
    wchar_t			                        requestName[ORC_NAME_LEN];
    wchar_t 		                        serverName[ORC_NAME_LEN];
    wchar_t			                        serverUserId[ORC_NAME_LEN];
    wchar_t			                        timeStarted[ORC_DATE_LEN];             // Using a String to hold timeStarted...
    wchar_t			                        timeFinished[ORC_DATE_LEN];            // Using a String to hold timeFinished...
    wchar_t			                        accessDefinitionName[ORC_NAME_LEN];
    wchar_t			                        accessDefinitionModificationDate[ORC_DATE_LEN];  // Storing this as a String on both sides
    wchar_t			                        extractFile[ORC_NAME_LEN];
    wchar_t			                        compareFile[ORC_NAME_LEN];
    wchar_t			                        controlFile[ORC_NAME_LEN];
    OptimReportingMessage_t               *pMessages;
    OptimReportingNameValuePair_t         *pRequestOptions;
    OptimReportingNumberPair_t            *pExtendedObjects;
    OptimReportingEntity_t	               *pEntities;
    OptimReportingTableImpAnalysis_t	   *pAnalysis;
    OptimReportingJoinControl_t           *pJoinControl;    
    OptimReportingRelationship_t          *pRelationships;
    OptimReportingStatistics_t	         *pStatistics;  
    unsigned int 	                        numMessages;               // Number of messages in messages array
    unsigned int 	                        numRequestOptions;         // Number of NV pairs in requestOptions array
    unsigned int 	                        numExtendedObjects;        // Number of pairs in extendedObjects array
    unsigned int	                        numEntities;               // Number of entities in entities array
    unsigned int	                        numImpactAnalysis;         // Number of impact Analysis in ImpactAnalysis array
    unsigned int	                        numRelationships;          // Number of relationships in relationships array
    unsigned int	                        numStatistics;             // Number of statistics items in statistics array
    unsigned int	                        numJoinControl;            // Number of joinControl items in statistics array
    int				                        accessDefinitionType;
    int 			                           reportType;
    wchar_t                               ddl[ORC_MAX_LEN];                                     
    wchar_t                               tableMapName[ORC_NAME_LEN];                            
    wchar_t                               dataSource1[ORC_NAME_LEN];                             
    wchar_t                               dataSource2[ORC_NAME_LEN];                             
    int                                   nonEqualSets;                                
    unsigned char                         singleTable;                                
    int                                   sourceType;                                  
    int                                   sourceType2;                                 
    int                                   totalSets;                                   
    int                                   numberOfArchiveFiles;                        
    int                                   numberOfSubsetFiles;                         
    int                                   numberOfFilesRestored;                       
    wchar_t                               reportName[ORC_NAME_LEN];                              
    wchar_t                               reportRole[ORC_NAME_LEN];                             
    wchar_t                               reportState[ORC_NAME_LEN];                             
    unsigned char                         importImported;                             
    wchar_t                               fileName[ORC_NAME_LEN];                                
    long long                             fileSize;   
};
typedef struct optimReportingContext OptimReportingContext_t;

/*-----------------------------------------------------------------------------
 * OptimReportingContext class
 *----------------------------------------------------------------------------*/
class OptimReportingContext_EXPORT OptimReportingContext
{  

   public:

         enum ActionRequest  {ArchiveAction = 0 ,                   // Action request types
                              ConvertAction = 1 ,
                              CompareAction = 2 ,
                              DeleteAction  = 3 ,
                              ExtractAction = 4 ,
                              InsertAction  = 5,
                              LoadAction    = 6,
                              RestoreAction = 7,
                              ReportAction  = 8,
                              Import  = 9,
                              AFMF    = 10,                    // FMF    
                              DIRMProcess = 11,                // Directory Maintenance Facility
                              ARCMProcess = 12                 // Arc Maintenance Facility
                    };

          enum ReportType  {  Archive = 0 ,                   // Report types                              
                              Compare = 1 ,
                              Convert = 2 ,
                              Delete  = 3 ,
                              Extract = 4 ,
                              Insert  = 5,
                              Load    = 6,
                              Report = 7,
                              Restore  = 8,
                              Utility = 9                              
                    };
        
          enum DBMS {   ORACLE = 1,                      // Oracle 
                        SYBASE = 2,                      // Sybase 
                        ODBC = 3,                        // ODBC 
                        INFORMIX = 4,                    // Informix 
                        DB2ZOS = 5,                      // DB2 zOS  
                        DB2LUW = 6,                      // DB2 LUW 
                        SQLSERVER = 7,                   // MS SQL Server
                        TERADATA = 8,			            // Teradata
                        NETEZZA	= 9		               // Netezza
                    }; 

          enum MessageType { Info = 0,
                             Warning = 1,
                             Error = 2,                            
                             Exit = 3
                           };

          enum RequestOptions {  Data,
                                 Objects,
                                 Both
                              };          

          enum ExtObjectType { Alias = 0,
                               Assembly = 1,
                               Default = 2,
                               Function = 3,
                               Index = 4,
                               Method = 5,
                               Package =6,
                               PartitionFunction = 7,
                               PartitionSchema = 8,
                               PrimaryKey = 9,
                               Procedure = 10,
                               Relationship = 11,
                               Rule = 12,
                               Sequence = 13,
                               Synonmns = 14,
                               Trigger = 15,
                               Udt = 16,
                               Views = 17,
                               ExtendedObject = 18
                         };

          enum EntityType {  Table = 0,                             
                             View = 1,
                             Alia = 2,
                             Synonym = 3
                        };

          enum AccessStrategy { NO_KEYS =       0,       // No Keys to Lookup
                                ONLY_1_KEY =    1,       // Only one Key value to Lookup
                                FORCE_SCAN =    2,       // User Forced Scan
                                FORCE_LOOKUP =  3,       // User Forced Key Lookup
                                LOOKUP  =       4,       // KeyLookup - All Keys indexed
                                SCAN_NO_INDEX =  5,      // Scan - No Index
                                SCAN_TOO_MANY_KEYS =  6, // Scan - Too many keys to lookup
                                LOOKUP_TOO_FEW_KEYS = 7, // KeyLookup - Too few keys to scan
                                SCAN_SUBQUERY  =      8, // Scan with temp table subquery
                                FORCE_SCAN_SUBQUERY = 9  // User Forced Scan with subquery
                              };

        enum DBMSAccessType {  Cursor_Scan = 0,        // Single Cursor scan
                               PK_lookup = 1,          // PK lookup
                               FK_lookup = 2           // FK lookup                           
                            };
   
        enum compressionType { Yes = 0,                 // File compression
                               No = 1
                             };

        enum RelDirectionType { Parent = 0,             // Parent
                                Dependent = 1           // Child
                              };

        enum RelAccessType { FORCE_NONE = 0,              // Not Forcing
                             RELFORCE_SCAN = 1,           // Force Scan
                             FORCE_KEY = 2,               // Force Key
                             FORCE_SCANSUBQUERY = 3,      // Force ScanSubquery
                             ACCESS_NONE = 4,             // Access None
                             ACCESS_KEY = 5,              // Access Keys
                             ACCESS_SCAN = 6,             // Access Scan
                             ACCESS_SCANSUBQUERY = 7     // Access ScanSubquery                                              
                           };

         enum SourceType { SourceTypeInvalid,
                           SourceTypeExtractFile,
                           SourceTypeDatabaseTable,
                           SourceTypeAccessDefinition,
                           SourceTypeAllTables,
                           SourceTypeArchiveFile
                         };

         enum ObjectCreation { Succedded = 0,
                               Partial = 1,
                               Failed = 2
                              };

      
      /*-------------------------------------------------------------------
       * Standard dialog constructor
       *-------------------------------------------------------------------*/
       OptimReportingContext();

       /*-------------------------------------------------------------------
       * Standard dialog destructor
       *-------------------------------------------------------------------*/
       virtual ~OptimReportingContext();

       private:
         // Data members
         ActnPort        *mpActnPort;     
         RtErrors         mErrors;         
         int              mNumTables;
         int              mActionType;
         int              mErrorCode;
         CString          mActionName;
         CString          mParActionName;
         CString          mDBMSType;
         CString          mDBMSVer;
         CString          mCompression;
         CString          mFileName;
         CString          mHtmlFileName;                  
         CString          mFullName;
         CString          mConnMode;
         CString          mRelName;
         CString          mORCRec;
         RtUnicodeFile    mORCFile;  
         DWORD            mFlags;           //  General Flags      
         BOOL             mRequest;
         OptimReportingContext_t *mpContext;	               
         CStringArray     mExtObject;
         CStringArray     mExtType; 
         CString          mSQLText;
         CString          mRowId;         

      protected:
         ActnPort *GetActionPortable(){return mpActnPort;}

       public:

      /*-------------------------------------------------------------------
       * OptimReportingConcept  Report Gen Methods
       *-------------------------------------------------------------------*/
          BOOL BuildOptimReportingContext(ActnPort *pActnPort, int ActionType, CString ActionName, 
                                          BOOL bRequest, CString ParAction);
          BOOL BuildCommonSection();
          BOOL BuildDefaults();
          BOOL BuildEntity();
          BOOL BuildExtendedObjects();
          BOOL BuildImpactAnalysis();
          BOOL BuildJoinControl();
          BOOL BuildCommonErrorMessages();
          BOOL BuildCommonWarnMessages();
          BOOL BuildCommonInfoMessages();
          BOOL BuildCommonExitMessages();
          BOOL BuildRequestOptions();
          BOOL BuildRelationship();
          BOOL BuildStatistics();
          BOOL WriteOptimDirectory();
          BOOL GetOptimDBInfo ();
          BOOL GetOptimDBMSVer(int DBMSVer); 
          BOOL PopulateOptimReportingContext();
          // Action Request Reports       
          int GetSourceType( short sType, int source );
          CString GetSource1Name( short sType );
          CString GetSource2Name( short sType );  
};

#endif // OptimReportingContext_H
