<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:optim="http://www.ibm.com/optim/audit/v11.3.0.0"
	xmlns:optimreport="http://www.ibm.com/optim/report/v11.3.0.0"
	xmlns:optimcommon="http://www.ibm.com/optim/common/v11.3.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">

	<xsl:import href="CommonTemplates.xsl"/>
	<xsl:output method="html" indent="yes" version="4.0"
		doctype-public="-//W3C//DTD HTML 4.01//EN" doctype-system="http://www.w3.org/TR/html4/strict.dtd" />

	<!-- BEGIN entities section -->
	<xsl:template name="insertEntities">
		<table class="entity_details noBorder">
			<xsl:for-each select="optimreport:entities/optimreport:entity">
				<!-- Table Title -->
				<tr class="alternativeTitle" isTitle="yes">
					<xsl:choose>
						<xsl:when test="position() = 1 or position() mod 20 = 0">
							<xsl:attribute name="style"></xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="style">display:none</xsl:attribute>
							<xsl:attribute name="inallaction">true</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
					<td>
					</td>
					<td>
						<center>
							<xsl:text>Inserted</xsl:text>
						</center>
					</td>
					<td>
						<center>
							<xsl:text>Updated</xsl:text>
						</center>
					</td>
					<td>
						<center>
							<xsl:text>Skipped</xsl:text>
						</center>
					</td>
					<td>
						<center>
							<xsl:text>Failed</xsl:text>
						</center>
					</td>
				</tr>
				<xsl:variable name="rowid">
					<xsl:value-of select="generate-id()"/>
				</xsl:variable>

				<xsl:variable name="childrenRowsId">
					<xsl:value-of select="generate-id(optimreport:maskedColumns)"/>
				</xsl:variable>
				<tr class="rightAlign" swaparrow="true">
					<th class="drop_down_hover category_title entity_rows" onClick="toggleRows('{$rowid}', event); SwapByClass('blockArrow', '{$rowid}', event); SwapChildrenElement('blockArrow', 'entity_details', '{$childrenRowsId}', '{$rowid}', event); ShowNextTitle('alternativeTitle', '{$rowid}', event);">
						<xsl:variable name="numberOfClassifiedUnmaskedColumns">
							<xsl:value-of select="count(optimreport:unmaskedColumns/optimcommon:column/optimcommon:privacyInformation/optimcommon:classification[@label and @label!=''])" />
						</xsl:variable>
						<xsl:variable name="numberOfEmptymaskedColumns">
							<xsl:value-of select="count(optimreport:maskedColumns/optimcommon:column/optimcommon:privacyInformation/optimcommon:classification[not(@label) or @label=''])" />
						</xsl:variable>
						<xsl:variable name="numberOfmaskedColumns">
							<xsl:value-of select="count(optimreport:maskedColumns/optimcommon:column)"/>
						</xsl:variable>
						<xsl:variable name="description">
							<xsl:choose>
								<xsl:when test="number($numberOfmaskedColumns)">
									<xsl:choose>
										<xsl:when test="(number($numberOfClassifiedUnmaskedColumns) + number($numberOfEmptymaskedColumns)) &lt; 2">
											<xsl:text>Data in this table was masked according to the specified privacy policy of one or more columns, and one of the columns in this table has warning message.</xsl:text>
										</xsl:when>
										<xsl:otherwise>
											<xsl:text>Data in this table was masked according to the specified privacy policy of one or more columns, and </xsl:text>
											<xsl:value-of select="$numberOfClassifiedUnmaskedColumns + $numberOfEmptymaskedColumns"/>
											<xsl:text> of the columns in this table has warning messages.</xsl:text>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:when>
								<xsl:otherwise>
									<xsl:choose>
										<xsl:when test="(number($numberOfClassifiedUnmaskedColumns) + number($numberOfEmptymaskedColumns)) &lt; 2">
											<xsl:text>One of the columns in this table has warning message.</xsl:text>
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="$numberOfClassifiedUnmaskedColumns + $numberOfEmptymaskedColumns"/>
											<xsl:text> of the columns in this table has warning messages.</xsl:text>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<xsl:choose>
							<!-- If has warnings -->
							<xsl:when test="(number($numberOfClassifiedUnmaskedColumns) + number($numberOfEmptymaskedColumns))">
								<xsl:call-template name="setImage">
									<xsl:with-param name="class">infopop_warning_class</xsl:with-param>
									<xsl:with-param name="name">warning</xsl:with-param>
									<xsl:with-param name="URI">
										<xsl:choose>
											<xsl:when test="number($numberOfmaskedColumns)">
												<xsl:value-of select="$ribbonWarning_URI"/>
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of select="$warning_URI"/>
											</xsl:otherwise>
										</xsl:choose>
									</xsl:with-param>
									<xsl:with-param name="description" select="$description"/>
								</xsl:call-template>

								<xsl:call-template name="popupBlock">
									<xsl:with-param name="class">warning_pop_class</xsl:with-param>
									<xsl:with-param name="description" select="$description"/>
								</xsl:call-template>
							</xsl:when>
							<!-- If has masked columns and no warnings-->
							<xsl:when test="number($numberOfmaskedColumns)">
								<xsl:call-template name="setImage">
									<xsl:with-param name="class">infopop_warning_class</xsl:with-param>
									<xsl:with-param name="name">warning</xsl:with-param>
									<xsl:with-param name="URI">
										<xsl:value-of select="$medal_URI"/>
									</xsl:with-param>
									<xsl:with-param name="description" select="$medalDescription"/>
								</xsl:call-template>

								<xsl:call-template name="popupBlock">
									<xsl:with-param name="class">infopop_warning_class</xsl:with-param>
									<xsl:with-param name="description" select="$medalDescription"/>
								</xsl:call-template>
							</xsl:when>
						</xsl:choose>
						<!--Table name-->
						<xsl:call-template name="arrow">
							<xsl:with-param name="id" select="concat($rowid,'_arrow')"/>
							<xsl:with-param name="isExpand">false</xsl:with-param>
						</xsl:call-template>
						<span class="main_title">
							<xsl:call-template name="stringCheck">
								<xsl:with-param name="value" select="@name"/>
							</xsl:call-template>
						</span>
					</th>
					<td align="right">
						<xsl:choose>
							<xsl:when test='number(@inserted) = number(@inserted)'>
								<xsl:call-template name="numberFormat">
									<xsl:with-param name="number" select="@inserted"/>
									<xsl:with-param name="digitsAfterPoint">0</xsl:with-param>
								</xsl:call-template>
							</xsl:when>
							<xsl:otherwise>
								<xsl:call-template name='stringCheck'>
									<xsl:with-param name='value' select='@inserted'/>
								</xsl:call-template>
							</xsl:otherwise>
						</xsl:choose>
					</td>
					<td align="right">
						<xsl:choose>
							<xsl:when test='number(@updated) = number(@updated)'>
								<xsl:call-template name="numberFormat">
									<xsl:with-param name="number" select="@updated"/>
									<xsl:with-param name="digitsAfterPoint">0</xsl:with-param>
								</xsl:call-template>
							</xsl:when>
							<xsl:otherwise>
								<xsl:call-template name='stringCheck'>
									<xsl:with-param name='value' select='@updated'/>
								</xsl:call-template>
							</xsl:otherwise>
						</xsl:choose>
					</td>
					<td align="right">
						<xsl:choose>
							<xsl:when test='number(@skipped) = number(@skipped)'>
								<xsl:call-template name="numberFormat">
									<xsl:with-param name="number" select="@skipped"/>
									<xsl:with-param name="digitsAfterPoint">0</xsl:with-param>
								</xsl:call-template>
							</xsl:when>
							<xsl:otherwise>
								<xsl:call-template name='stringCheck'>
									<xsl:with-param name='value' select='@skipped'/>
								</xsl:call-template>
							</xsl:otherwise>
						</xsl:choose>
					</td>
					<td align="right">
						<xsl:choose>
							<xsl:when test='number(@failed) = number(@failed)'>
								<xsl:call-template name="numberFormat">
									<xsl:with-param name="number" select="@failed"/>
									<xsl:with-param name="digitsAfterPoint">0</xsl:with-param>
								</xsl:call-template>
							</xsl:when>
							<xsl:otherwise>
								<xsl:call-template name='stringCheck'>
									<xsl:with-param name='value' select='@failed'/>
								</xsl:call-template>
							</xsl:otherwise>
						</xsl:choose>
					</td>
				</tr>
				
				<!-- Column Information -->
				<xsl:call-template name="columnMap">
					<xsl:with-param name="rowid" select="$rowid"/>
					<xsl:with-param name="childrenRowsId" select="$childrenRowsId"/>
					<xsl:with-param name="warning_URI" select="$warning_URI"/>
					<xsl:with-param name="information_URI" select="$information_URI"/>
				</xsl:call-template>

				<xsl:if test="not(position() = last())">
					<tr class="hiddenRow">
						<td colspan="4"></td>
					</tr>
				</xsl:if>
			</xsl:for-each>
		</table>
	</xsl:template>
	<!-- END entities section -->

	<!--BEGIN Statistics Table-->
	<xsl:template name="statisticTables">
		<xsl:param name="nodeName"/>
		<table class="{$nodeName}_table resizetable">
			<xsl:for-each select="*">
				<xsl:variable name="rowId" select="generate-id()"/>
				<tr swaparrow="true" isTitle="yes">
					<th class="category_title drop_down_hover" onClick="toggleRows('{$rowId}', event); SwapByClass('blockArrow', '{$rowId}', event)">
						<xsl:choose>
							<xsl:when test="(@numberOfTables and @numberOfTables &gt; 0) or (@numberOfColumns and @numberOfColumns &gt; 0)">
								<xsl:call-template name="arrow">
									<xsl:with-param name="isExpand">false</xsl:with-param>
								</xsl:call-template>
								<span class="second_indent">
									<xsl:call-template name="stringCheck">
										<xsl:with-param name="value" select="@label"/>
									</xsl:call-template>
								</span>
							</xsl:when>
							<xsl:otherwise>
								<span class="second_indent">
									<xsl:call-template name="stringCheck">
										<xsl:with-param name="value" select="@label"/>
									</xsl:call-template>
								</span>
							</xsl:otherwise>
						</xsl:choose>
					</th>
					<td>
						<center>
							<xsl:text>Tables: </xsl:text>
							<xsl:choose>
								<xsl:when test="@numberOfTables = @numberOfTables">
									<xsl:call-template name="numberFormat">
										<xsl:with-param name="number" select="@numberOfTables"/>
										<xsl:with-param name="digitsAfterPoint">0</xsl:with-param>
									</xsl:call-template>
								</xsl:when>
								<xsl:otherwise>
									0
								</xsl:otherwise>
							</xsl:choose>
						</center>
					</td>
					<td>
						<center>
							<xsl:text>Columns: </xsl:text>
							<xsl:choose>
								<xsl:when test="@numberOfColumns = @numberOfColumns">
									<xsl:call-template name="numberFormat">
										<xsl:with-param name="number" select="@numberOfColumns"/>
										<xsl:with-param name="digitsAfterPoint">0</xsl:with-param>
									</xsl:call-template>
								</xsl:when>
								<xsl:otherwise>
									0
								</xsl:otherwise>
							</xsl:choose>
						</center>
					</td>
				</tr>
				<xsl:choose>
					<xsl:when test="(@numberOfTables and @numberOfTables &gt; 0) or (@numberOfColumns and @numberOfColumns &gt; 0)">
						<xsl:for-each select="optimreport:qualifiedMaskedColumnName">
							<tr class="{$rowId}" style="display:none" inAllAction="true">
								<td class="emptyCell">

								</td>
								<td colspan="1">
									<xsl:call-template name="stringCheck">
										<xsl:with-param name="value">
											<xsl:call-template name="substring-last">
												<xsl:with-param name="string" select="./text()"/>
												<xsl:with-param name="char" select="'.'"/>
												<xsl:with-param name="direction">before</xsl:with-param>
												<xsl:with-param name="resultString"></xsl:with-param>
											</xsl:call-template>
										</xsl:with-param>
									</xsl:call-template>
								</td>
								<td colspan="1">
									<xsl:call-template name="stringCheck">
										<xsl:with-param name="value">
											<xsl:call-template name="substring-last">
												<xsl:with-param name="string" select="./text()"/>
												<xsl:with-param name="char" select="'.'"/>
												<xsl:with-param name="direction">after</xsl:with-param>
												<xsl:with-param name="resultString"></xsl:with-param>
											</xsl:call-template>
										</xsl:with-param>
									</xsl:call-template>
								</td>
							</tr>
						</xsl:for-each>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>

		</table>
	</xsl:template>
	<!--END Statistics Table-->
	
	<!-- BEGIN STRING LAST INDEX -->
	<xsl:template name="substring-last">
		<xsl:param name="string"/>
		<xsl:param name="char"/>
		<xsl:param name="direction"/>
		<xsl:param name="resultString"/>
		<xsl:choose>
			<xsl:when test="contains($string, $char)">
				<xsl:call-template name="substring-last">
					<xsl:with-param name="string" select="substring-after($string, $char)"/>
					<xsl:with-param name="char" select="$char"/>
					<xsl:with-param name="direction" select="$direction"/>
					<xsl:with-param name="resultString">
						<xsl:choose>
							<xsl:when test="$direction = 'before'">
								<xsl:choose>
									<xsl:when test="string-length($resultString) = 0">
										<xsl:value-of select="concat($resultString, substring-before($string, $char))"/>
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="concat(concat($resultString, $char), substring-before($string, $char))"/>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="$resultString"/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:choose>
					<xsl:when test="$direction='before'">
						<xsl:value-of select="$resultString"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$string"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- END STRING LAST INDEX -->

</xsl:stylesheet>